%option noyywrap

%{

#define YY_DECL int yylex()

#include "calculator.hpp"
#include "book.tab.hpp"

%}

decimal     [0-9]+
binary      0[Bb][01]+
octal       0[Oo][0-7]+
hexadecimal 0[Xx][0-9A-Fa-f]+
floating    (({decimal}?\.{decimal}+)|{decimal}\.?)([Ee][+-]?{decimal})?

%%

{decimal}           { yylval.str = yytext; return DECIMAL; }
{binary}            { yylval.str = yytext; return BINARY; }
{octal}             { yylval.str = yytext; return OCTAL; }
{hexadecimal}       { yylval.str = yytext; return HEXADECIMAL; }
{floating}          { yylval.str = yytext; return FLOATING; }
[Pp][Ii]            { yylval.str = yytext; return PI; }
[Ee]                { yylval.str = yytext; return E; }
[Ss][Ii][Nn]        { yylval.str = yytext; return SIN; }
[Cc][Oo][Ss]        { yylval.str = yytext; return COS; }
[Tt][Aa][Nn]        { yylval.str = yytext; return TAN; }
[Aa][Ss][Ii][Nn]    { yylval.str = yytext; return ASIN; }
[Aa][Cc][Oo][Ss]    { yylval.str = yytext; return ACOS; }
[Aa][Tt][Aa][Nn]    { yylval.str = yytext; return ATAN; }
[Ll][Oo][Gg]        { yylval.str = yytext; return LOG; }
[Ll][Nn]            { yylval.str = yytext; return LN; }

[\+\-\*\/\(\)\n]    { yylval.str = yytext; return *yytext; }
\*\*                { yylval.str = yytext; return POWER; }
[ \t\r\x0b\x0c]     {}
.                   { return ERROR; }

%%
