%{

#include "calculator.hpp"

extern int yylex();
extern int yyparse();
void yyerror(const char *s) {}

%}

%union {
    const char *str;
    NodeExp *node;
};

/*终结符*/
%token <str> DECIMAL
%token <str> BINARY
%token <str> OCTAL
%token <str> HEXADECIMAL
%token <str> FLOATING
%token <str> POWER
%token <node> PI
%token <node> E
%token <node> SIN
%token <node> COS
%token <node> TAN
%token <node> ASIN
%token <node> ACOS
%token <node> ATAN
%token <node> LOG
%token <node> LN
%token <str> ERROR

/*优先级结合性*/
%left '+' '-'
%left '*' '/'
%right POWER

%type <node> exp

%%

input   : /* empty string */
        | input line
        ;
line    : '\n'
        | exp '\n'          { draw($1); delete $1; }
        | error '\n'        { draw_error(); }
        ;
exp     : FLOATING          { $$ = new NodeExp(OperationType::Floating, $1); }
        | DECIMAL           { $$ = new NodeExp(OperationType::Decimal, $1); }
        | BINARY            { $$ = new NodeExp(OperationType::Binary, $1); }
        | OCTAL             { $$ = new NodeExp(OperationType::Octal, $1); }
        | HEXADECIMAL       { $$ = new NodeExp(OperationType::Hexadecimal, $1); }
        | PI                { $$ = new NodeExp(OperationType::PI); }
        | E                 { $$ = new NodeExp(OperationType::E); }
        | exp '+' exp       { $$ = new NodeExp(OperationType::Add, {$1, $3}); }
        | exp '-'  exp      { $$ = new NodeExp(OperationType::Sub, {$1, $3}); }
        | exp '*'  exp      { $$ = new NodeExp(OperationType::Mul, {$1, $3}); }
        | exp '/'  exp      { $$ = new NodeExp(OperationType::Div, {$1, $3}); }
        | exp POWER exp     { $$ = new NodeExp(OperationType::Power, {$1, $3}); }
        | '+' exp           { $$ = new NodeExp(OperationType::Add, {$2}); }
        | '-' exp           { $$ = new NodeExp(OperationType::Sub, {$2}); }
        | '(' exp ')'       { $$ = new NodeExp(OperationType::Brackets, {$2}); }
        | SIN  '(' exp ')'  { $$ = new NodeExp(OperationType::Sin, {$3}); }
        | COS  '(' exp ')'  { $$ = new NodeExp(OperationType::Cos, {$3}); }
        | TAN  '(' exp ')'  { $$ = new NodeExp(OperationType::Tan, {$3}); }
        | ASIN '(' exp ')'  { $$ = new NodeExp(OperationType::Asin, {$3}); }
        | ACOS '(' exp ')'  { $$ = new NodeExp(OperationType::Acos, {$3}); }
        | ATAN '(' exp ')'  { $$ = new NodeExp(OperationType::Atan, {$3}); }
        | LOG  '(' exp ')'  { $$ = new NodeExp(OperationType::Log, {$3}); }
        | LN   '(' exp ')'  { $$ = new NodeExp(OperationType::Ln, {$3}); }
        ;

%%

/*用户函数*/
int main(){
    printf("graph \"book.y\" {\n"
           "    node [fontname = courier, shape = box, colorscheme = paired6]\n"
           "    edge [fontname = courier]\n");

    yyparse();

    printf("    \n"
           "}\n");

    return 0;
}
